# coding=utf-8
import logging
import os
import threading
from builtins import *
from logging.handlers import RotatingFileHandler

logger = logging.getLogger(__file__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(module)s - %(message)s')
handler = RotatingFileHandler('log.log', maxBytes=100 * 1024 * 1024, backupCount=5)
handler.setFormatter(formatter)
logger.addHandler(handler)


class GlobalData(object):
    suspend_event = threading.Event()  # 暂停,默认值是False，False是暂停，True是运行中
    stop_event = threading.Event()  # 停止，默认是False，False 是停止， True 是运行中



# coding=utf-8
import os
import time
import traceback
from builtins import *
import datetime
from multiprocessing.context import Process

from performancetest.core.base.actuator import Actuator
from performancetest.core.cpu import CpuMonitor
from performancetest.core.device import AndroidDevice, IosDevice
from performancetest.core.devicebattery import DeviceBatteryMonitor
from performancetest.core.fps import FPSMonitor
from performancetest.core.global_data import GlobalData as G, logger
from performancetest.core.gpu import GpuMonitor
from performancetest.core.iosperf import IosPerfMonitor
from performancetest.core.memory import MemoryMonitor
from performancetest.core.snapshot import SnapshotMonitor, IosSibSnapshotMonitor
from performancetest.web.dao import connect, Task


class TaskHandle(Process, Actuator):

    def __init__(self, serialno: str, server_addr: list[str], package: str, save_dir: str, task_id: int,
                 device_platform: str):
        super(TaskHandle, self).__init__()
        self.serialno = serialno
        self.server_addr = server_addr
        self.package = package
        self.save_dir = save_dir
        if not os.path.exists(self.save_dir):
            os.makedirs(self.save_dir)
        self.daemon = True
        self.task_id = task_id
        self.device_platform = device_platform  # ios | android

    def start(self):
        logger.info("join task handle")
        super().start()

    def run(self):
        logger.info("join task handle run")
        with connect() as session:
            current_task_running = session.query(Task).filter(
                Task.id == self.task_id).first()
            if current_task_running:
                current_task_running.status = 1
                current_task_running.pid = self.pid
            else:
                raise Exception("任务不存在")
            if self.device_platform == "android":
                try:
                    G.device = AndroidDevice(serialno=self.serialno, server_addr=self.server_addr,
                                             package=self.package, save_dir=self.save_dir)
                    G.device.start_app()
                except:
                    traceback.print_exc()
                    current_task_running.status = -1
                    session.flush()
                    session.commit()
                    raise Exception("任务启动失败")
                time.sleep(0.1)
                CpuMonitor(os.path.join(self.save_dir, "cpu.csv")).start()
                MemoryMonitor(os.path.join(self.save_dir, "memory.csv")).start()
                FPSMonitor(os.path.join(self.save_dir, "fps.csv")).start()
                GpuMonitor(os.path.join(self.save_dir, "gpu.csv")).start()
                DeviceBatteryMonitor(os.path.join(self.save_dir, "devicebattery.csv")).start()
                SnapshotMonitor(os.path.join(self.save_dir, "picture_log"), self.serialno, self.server_addr).start()
            elif self.device_platform == "ios":
                try:
                    G.device = IosDevice(serialno=self.serialno, device_addr=self.server_addr,
                                         package=self.package, save_dir=self.save_dir)
                    G.device.start_app()
                except:
                    traceback.print_exc()
                    current_task_running.status = -1
                    current_task_running.end_time = datetime.datetime.now()
                    session.flush()
                    session.commit()
                    raise Exception("任务启动失败")
                IosPerfMonitor(save_dir=self.save_dir, G=G).start()
                IosSibSnapshotMonitor(os.path.join(self.save_dir, "picture_log"), G.device, self.package).start()

    def stop(self):
        G.stop_event.clear()

    def suspend(self):
        G.suspend_event.clear()


if __name__ == '__main__':
    task_process = TaskHandle(serialno="00008110-0012148E1E8B801E", server_addr=["10.131.129.128", 9123],
                              package="com.netease.id5", save_dir="localhost", task_id=1, device_platform="ios")
    task_process.start()
    time.sleep(2 * 10)



# coding=utf-8
import threading
import time
from builtins import *
from pathlib import Path
from airtest.core.android import Android
from func_timeout import func_set_timeout, FunctionTimedOut

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger


class SnapshotMonitor(Monitor):
    def __init__(self, save_dir, serialno, server_addr, interval=1, test_time=-1):
        super().__init__()
        self.save_dir = self.get_save_file(save_dir)
        self.interval = interval
        self.device = Android(serialno, host=server_addr)

    def get_save_file(self, save_dir):
        save_dir = Path(save_dir)
        if not save_dir.is_dir():
            save_dir.mkdir(parents=True, exist_ok=True)
        return save_dir

    def snapshot(self, name):
        self.device.snapshot(filename=name)

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        '''
        按照指定频率手机截图
        '''
        logger.info("开始保存图片")
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            try:
                logger.debug(
                    "---------------开始截图, _collect_screenshot loop thread is : " + str(threading.current_thread().name))
                before = time.time()
                self.snapshot(self.save_dir.joinpath(str(int(before * 1000)) + ".jpg"))
                after = time.time()
                time_consume = after - before
                logger.debug("  ============== time consume snapshot time : " + str(time_consume))
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in snapshot thread , reason unkown!, e:")
                logger.exception(e)
        logger.debug("snapshot stop")


class IosSibSnapshotMonitor(Monitor):
    def __init__(self, save_dir, device, package, interval=1, test_time=-1):
        super(IosSibSnapshotMonitor, self).__init__()
        self.save_dir = self.get_save_file(save_dir)
        self.interval = interval
        self.package = package
        self.device = device
        self.screenshot_collect = threading.Thread(target=self._collect_screenshot, args=(test_time,))
        self.running = False
        self.G = G

    def get_save_file(self, save_dir):
        save_dir = Path(save_dir)
        if not save_dir.is_dir():
            save_dir.mkdir(parents=True, exist_ok=True)
        return save_dir

    @func_set_timeout(10)
    def snapshot(self, name):
        self.device.screenshot(name)

    def start(self):
        self.screenshot_collect.start()

    def stop(self):
        logger.info("close")

    def _collect_screenshot(self, test_time):
        '''
        按照指定频率手机截图
        '''
        logger.info("开始保存图片")
        G = self.G
        G.stop_event.set()
        while G.stop_event.is_set():
            try:
                logger.debug(
                    "---------------开始截图, _collect_screenshot loop thread is : " + str(threading.current_thread().name))
                before = time.time()
                self.snapshot(self.save_dir.joinpath(str(int(before * 1000)) + ".jpg"))
                after = time.time()
                time_consume = after - before
                logger.debug("  ============== time consume snapshot time : " + str(time_consume))
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except (Exception, FunctionTimedOut) as e:
                logger.error("an exception hanpend in snapshot thread , reason unkown!, e:")
                logger.exception(e)
                time.sleep(0.5)
        logger.debug("snapshot stop")



# _*_ coding: utf-8 _*_
import os
from builtins import *
from contextlib import contextmanager

from sqlalchemy import create_engine, Column, String, Integer, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy_serializer import SerializerMixin

from performancetest.core.device import logger

logger.info("工作空间{0}".format(os.getcwd()))
db_path = os.path.join(os.getcwd(), "task.sqlite")
logger.info("db path {0}".format(db_path))
engine = create_engine('sqlite:///{0}'.format(db_path))
logger.info("current path {0}".format(os.getcwd()))
Session = sessionmaker(bind=engine)
Base = declarative_base()


@contextmanager
def connect():
    logger.info("begin sql")
    session = Session()
    try:
        yield session
        session.commit()
        logger.info("sql success")
    except Exception as e:
        session.rollback()
        raise e
    finally:
        session.close()
        logger.info("sql end")


class Task(Base, SerializerMixin):
    __tablename__ = 'tasks'
    id = Column(Integer, primary_key=True, autoincrement=True)
    host = Column(String(50), default=None)
    port = Column(Integer)
    start_time = Column(DateTime, default=None)
    end_time = Column(DateTime, default=None)
    serialno = Column(String(255), default=None)
    status = Column(Integer)  # 0未开始, 1 执行中 , 2 执行完成 3.暂停
    file_dir = Column(String(255), default=None)  # 存储csv文件的路径
    package = Column(String(255), default=None)  # 测试的app包名
    pid = Column(Integer)  # 当前任务运行的进程pid，任务执行的进程，里面有各个性能指标的线程
    platform = Column(String(50), default="android")  # android | ios 任务


Base.metadata.create_all(engine)



# _*_ coding: utf-8 _*_
import asyncio
import datetime
import hashlib
import multiprocessing
import os
import re
import shutil
import sys
import time
import traceback
from builtins import *

import psutil
from airtest.core.android.adb import ADB
from fastapi import FastAPI, Request
from starlette.responses import RedirectResponse, JSONResponse
from starlette.staticfiles import StaticFiles
from tidevice import Usbmux
from tidevice._device import Device

sys.path.insert(0, os.path.join(os.path.dirname(os.path.dirname(os.getcwd()))))
from performancetest.core.global_data import logger
from performancetest.core.task_handle import TaskHandle
from performancetest.web.dao import connect, Task
from performancetest.web.entity import DeviceEntity, PackageEntity, TaskEntity
from performancetest.web.util import DataCollect

app = FastAPI()
logger.info("工作空间{0}".format(os.getcwd()))
BASE_CSV_DIR = os.path.join(os.path.dirname(__file__), "test_result")
if not os.path.exists(BASE_CSV_DIR):
    os.mkdir(BASE_CSV_DIR)
app.mount("/static", StaticFiles(directory=BASE_CSV_DIR), name="static")


@app.middleware("http")
async def http_filter(request: Request, call_next):
    try:
        response = await call_next(request)
    except BaseException as e:
        traceback.print_exc()
        return JSONResponse(content={"code": 500, "msg": str(e)})
    return response


@app.get("/")
def index():
    return RedirectResponse(url="/static/index.html")


@app.get("/get_local_device/")
async def get_local_device(request: Request):
    async def real_func():
        client_host: str = request.client.host
        adb: ADB = ADB(server_addr=(client_host, 5037))
        devices: list = await asyncio.wait_for(asyncio.to_thread(adb.devices), 10)
        logger.info("devices {0}".format(devices))
        res_list: list = []
        for i in devices:
            adb.serialno = i[0]
            info: dict = await asyncio.wait_for(asyncio.to_thread(adb.get_device_info), 10)
            info["host"] = client_host
            info["port"] = 5037
            info["platform"] = "android"
            res_list.append(info)
        logger.info(res_list)
        return res_list

    return await asyncio.wait_for(real_func(), timeout=15)


@app.post("/get_local_device_packages/")
async def get_local_device_packages(request: Request, device: DeviceEntity):
    client_host: str = request.client.host
    adb: ADB = ADB(server_addr=(client_host, 5037), serialno=device.serialno)
    app_list: list = await asyncio.wait_for(asyncio.to_thread(adb.list_app, third_only=True), 10)
    logger.info(app_list)
    if not app_list:
        app_list = []
    return [{"name": i, "package": i} for i in app_list]


@app.post("/get_local_device_packages_version/")
async def get_local_device_packages_version(request: Request, package: PackageEntity):
    client_host: str = request.client.host
    adb: ADB = ADB(server_addr=(client_host, 5037), serialno=package.serialno)
    package_info = await asyncio.wait_for(asyncio.to_thread(adb.shell, ['dumpsys', 'package', package.package]), 10)
    matcher = re.search(r'versionName=(.*)', package_info)
    if matcher:
        version = matcher.group(1)
    else:
        version = ''
    return version


@app.get("/get_local_device_ios/")
async def get_local_device_ios(request: Request):
    res = []
    for i in Usbmux().device_udid_list():
        res.append({"serialno": i, "host": "localhost", "port": 5037, "platform": "ios"})
    return res


@app.post("/get_local_device_packages_ios/")
async def get_local_device_packages_ios(request: Request, device: DeviceEntity):
    d = Device(device.serialno)
    return [{"name": i.get("CFBundleName"), "package": i.get("CFBundleIdentifier"),
             "version": i.get("CFBundleShortVersionString")}
            for i in d.installation.iter_installed(app_type="User")]


@app.get("/get_all_task/")
def get_all_task(request: Request):
    with connect() as session:
        all_task = session.query(Task).order_by(Task.start_time.desc()).all()
        return [i.to_dict() for i in all_task]


@app.post("/run_task/")
def run_task(request: Request, task: TaskEntity):
    client_host: str = request.client.host
    port = task.port
    serialno = task.serialno
    package = task.package
    start_time = time.time()
    status = 0
    file_dir = os.path.join(BASE_CSV_DIR, client_host, str(int(start_time)))
    logger.info("任务路径{0}".format(file_dir))
    if not os.path.exists(file_dir):
        os.makedirs(file_dir)
    return_task_id = None
    with connect() as session:
        task_running_count = session.query(Task).filter(Task.host == client_host).filter(Task.serialno == task.serialno).filter(
            Task.status != 2).count()
        if task_running_count > 0:
            raise Exception("当前设备仍有任务在进行,无法创建新任务")
        new_task = Task(host=client_host, port=port, serialno=serialno, start_time=datetime.datetime.now(),
                        status=status, file_dir=file_dir, package=package, platform=task.platform)
        session.add(new_task)
        session.commit()
        run_all_monitor(serialno, [client_host, port], package, file_dir, new_task.id, task.platform)
        return_task_id = new_task.id
    return {"code": 200, "taskid": return_task_id}


def run_all_monitor(serialno, server_addr: list, package, save_dir, task_id, device_platform):
    task_process = TaskHandle(serialno=serialno, server_addr=server_addr, package=package, save_dir=save_dir,
                              task_id=task_id, device_platform=device_platform)
    task_process.start()
    task_process.join(timeout=3)


@app.get("/stop_task/")
def stop_task(request: Request, id: int):
    client_host: str = request.client.host
    with connect() as session:
        task_item = session.query(Task).filter(Task.id == id).filter(Task.host == client_host).first()
        if not task_item:
            return {"code": 500, "msg": "暂无权限"}
        try:
            proc = psutil.Process(task_item.pid)
            proc.kill()
        except Exception as e:
            logger.error(e)
            traceback.print_exc()
        task_item.status = 2
        task_item.end_time = datetime.datetime.now()
        try:
            if task_item.platform == "ios":
                t = Device(task_item.serialno)
                with t.connect_instruments() as ts:
                    logger.info('Stop...')
                    ts.close()
        except:
            traceback.print_exc()
    return {"code": 200}


@app.get("/delete_task/")
async def delete_task(request: Request, id: int):
    username = request.headers.get("username")
    client_host: str = request.client.host
    with connect() as session:
        item = session.query(Task).filter(Task.id == id).filter(Task.host == client_host).first()
        if item:
            session.delete(item)
        if os.path.exists(item.file_dir):
            try:
                shutil.rmtree(item.file_dir)
            except:
                traceback.print_exc()
    return {"code": 200}

@app.get("/get_task_status/")
def get_task_status(request: Request, id: int):
    client_host: str = request.client.host
    with connect() as session:
        if str(id) in ["1", "2"]:
            task_item = session.query(Task).filter(Task.id == id).first()
        else:
            task_item = session.query(Task).filter(Task.id == id).filter(Task.host == client_host).first()
        if not task_item:
            return {}
        return task_item.to_dict()


@app.get("/result/")
def result(request: Request, id: int):
    client_host: str = request.client.host
    with connect() as session:
        task_item = session.query(Task).filter(Task.id == id).filter(Task.host == client_host).first()
        if not task_item:
            return {"result": {}}
        file_dir = task_item.file_dir
        try:
            result = DataCollect.read_data_all(file_dir)
        except BaseException as e:
            logger.error(e)
            traceback.print_exc()
            return {"result": {}}
        return {"result": result}


@app.get("/get_username/")
async def get_username(request: Request):
    # 获取客户端的 IP 地址
    client_ip = request.client.host
    # 获取浏览器头中的 User-Agent
    user_agent = request.headers.get("user-agent", "")
    # 生成唯一标识符的原始数据
    raw_data = f"{client_ip}{user_agent}".encode("utf-8")
    # 使用哈希函数（如 SHA256）将原始数据转换为固定长度的哈希值
    hash_value = hashlib.md5(raw_data).hexdigest()
    return hash_value


if __name__ == "__main__":
    import uvicorn
    multiprocessing.freeze_support()
    # uvicorn.run(app, host="0.0.0.0", port=80, log_level="debug")
    uvicorn.run("performancetest.web.main:app", host="0.0.0.0", port=80, log_level="debug", workers=10, reload=True)
    logger.info("服务启动成功请访问: http://localhost:80")



# coding=utf-8

import csv
import os
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger


class DeviceBatteryFomatter(object):

    def __init__(self, device_battery):
        self.device_battery = device_battery
        self.property_info = self.device_battery.split(os.linesep)
        self.temperature = self.get_temperature()
        self.level = self.get_level()
        self.charge = self.get_charge()

    def get_temperature(self):
        return float(self.get_target_proerty("temperature")) * 0.1

    def get_level(self):
        return self.get_target_proerty("level")

    def get_charge(self):
        return str(self.get_target_proerty("status")) == "2" or self.get_target_proerty("USB powered") == "true"

    def get_target_proerty(self, target):
        try:
            for p in self.property_info:
                if target in p:
                    return p.strip().split(": ")[1]
            return None
        except Exception as e:
            logger.error(e)
            return None

    def __repr__(self):
        return str({"property": self.property_info})


class DeviceBatteryMonitor(Monitor):
    def __init__(self, save_file, test_time=-1, interval=1):
        super().__init__()
        self.save_file = save_file
        self.test_time = test_time
        self._stop_event = threading.Event()
        self.interval = interval

    def get_device_battery_info(self):
        device_battery_info = G.device.adb.raw_shell("dumpsys battery".format(G.device.serialno)).decode()
        device_battery_info = DeviceBatteryFomatter(device_battery_info)
        logger.info("获取到的设备信息是：{}".format(device_battery_info))
        return device_battery_info

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        '''
        按照指定频率，循环搜集设备温度的信息
        :return:
        '''
        device_battery_title = ["timestamp", "devicetemperature", "devicebatterylevel%", "charge"]
        device_battery_file = self.save_file
        with open(device_battery_file, 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(device_battery_title)
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            device_battery_info_list = []
            try:
                logger.debug("---------------开始获取手机温度 电量信息: " + str(
                    threading.current_thread().name))
                before = time.time()
                device_battery_info = self.get_device_battery_info()
                device_battery_info_list.append(before)
                device_battery_info_list.append(device_battery_info.temperature)
                device_battery_info_list.append(device_battery_info.level)
                device_battery_info_list.append(device_battery_info.charge)
                after = time.time()
                time_consume = after - before
                logger.debug("  ============== time consume for devicetemperature info : " + str(time_consume))
                logger.info(device_battery_file + "*****************************")
                with open(device_battery_file, 'a+', encoding="utf-8") as df:
                    csv.writer(df, lineterminator='\n').writerow(device_battery_info_list)
                    del device_battery_info_list[:]
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in devicebatteryinfo thread , reason unkown!, e:")
                traceback.print_exc()
                G.device.get_pid()
        logger.debug("memory stop event is set or timeout")



# coding=utf-8
import csv
import logging
import os
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import logger, GlobalData


class IosPerfMonitor(Monitor):
    def __init__(self, save_dir, G, test_time=-1, interval=1):
        super().__init__()
        self.save_dir = save_dir
        self.G = G
        self.test_time = test_time
        self.interval = interval
        self.property_all = {
            "cpu": (os.path.join(self.save_dir, "cpu.csv"), ["timestamp", "cpu%"]),
            "memory": (os.path.join(self.save_dir, "memory.csv"), ["timestamp", "memory"]),
            "fps": (os.path.join(self.save_dir, "fps.csv"),
                    ["timestamp", "FPS", "lag_number", "FPS_full_number", "jank_number", "big_jank_number"]),
            "gpu": (os.path.join(self.save_dir, "gpu.csv"), ["timestamp", "gpu%"]),
            "devicebattery": (os.path.join(self.save_dir, "devicebattery.csv"),
                              ["timestamp", "devicetemperature", "devicebatterylevel", "charge"])
        }
        for k, v in self.property_all.items():
            with open(v[0], 'w+') as df:
                logger.info('-------------------初始csv')
                csv.writer(df, lineterminator='\n').writerow(v[1])
        self.ios_perf_collect = [
            threading.Thread(target=self._collect_package_ios_perf_all_thread),
            threading.Thread(target=self._collect_ios_battery_thread)
        ]

    def get_perf_info(self):
        """
            target_o:  cpu, mem, gpu, fps
        """
        G = self.G
        res_line = G.device.get_perf()
        return res_line

    def get_battery(self):
        G = self.G
        battery_info = G.device.get_battery()
        battery_info_res = dict()
        battery_info_res['battery'] = battery_info['CurrentCapacity']
        battery_info_res['level'] = battery_info['Temperature'] / 100
        return battery_info_res

    def start(self):
        # to do 后续监控进程是否存在  来重启线程
        for th in self.ios_perf_collect:
            th.start()

    # 结束任务
    def stop(self):
        self.G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        self.G.suspend_event.clear()

    def _collect_package_ios_perf_all_thread(self):
        G = self.G
        G.stop_event.set()
        G.suspend_event.set()
        for k, v in self.property_all.items():
            if k == "devicebattery":
                continue
            with open(v[0], 'w+') as df:
                csv.writer(df, lineterminator='\n').writerow(v[1])
        while G.stop_event.is_set():
            try:
                logging.debug("---------------开始获取所有性能信息" + str(
                    threading.current_thread().name))
                before = time.time()
                perf_line_list = self.get_perf_info()  # 可超时的任务
                if not perf_line_list:
                    time.sleep(0.2)
                    continue
                for perf_line in perf_line_list:
                    logger.info("perf result {0}".format(perf_line))
                    logging.info("---iosperf 获取到数据 {0}".format('performance: ', perf_line))
                    print(type(perf_line), perf_line)
                    file_path = self.property_all.get(perf_line.get("type"))
                    if not file_path:
                        time.sleep(0.2)
                        continue
                    values = perf_line.get("value")
                    value_list = [values.get("timestamp") // 1000, str(values.get("value"))]
                    with open(file_path[0], 'a+', encoding="utf-8") as df:
                        logging.info("write {0}".format(value_list))
                        csv.writer(df, lineterminator='\n').writerow(value_list)
                        del value_list[:]
                    logger.info("value all: {0}".format(perf_line))
                after = time.time()
                time_consume = after - before
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logging.error("an exception hanpend in ios perf thread , reason unkown!, e: iosperf")
                logging.error(e)
                logging.error(traceback.format_exc())
                after = time.time()
                time_consume = after - before
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
        logging.debug("stop event is set or timeout iosperf")

    def _collect_ios_battery_thread(self):
        G = self.G
        G.stop_event.set()
        G.suspend_event.set()
        with open(self.property_all.get("devicebattery")[0], 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(self.property_all.get("devicebattery")[1])
        while G.stop_event.is_set():
            try:
                logging.debug("---------------开始获取battery性能信息" + str(
                    threading.current_thread().name))
                before = time.time()
                battery_dict = self.get_battery()  # 可超时的任务
                if not battery_dict:
                    time.sleep(0.2)
                    continue
                write_info = []
                write_info.append(int(time.time()))
                write_info.append(battery_dict.get("level"))
                write_info.append(battery_dict.get("battery"))
                write_info.append("true")
                with open(self.property_all.get("devicebattery")[0], 'a+', encoding="utf-8") as df:
                    logging.info("write devicebattery {0}".format(write_info))
                    csv.writer(df, lineterminator='\n').writerow(write_info)
                    del write_info[:]
                after = time.time()
                time_consume = after - before
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logging.error("an exception hanpend in ios perf thread , reason unkown!, e: iosperf")
                logging.error(e)
                logging.error(traceback.format_exc())
        logging.debug("stop event is set or timeout iosperf")



# coding=utf-8

import csv
import os
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger


class ParseCpuinfo(object):
    def __init__(self, package, cpuinfo, sdkversion=None):
        self.cpuinfo = cpuinfo
        self.package = package
        self.sdkversion = sdkversion
        self.cpur_rate = self.get_cpu_rate()

    def get_cpu_rate(self):
        for pidinfo in self.cpuinfo.split(os.linesep):
            if self.package in pidinfo:
                pidinfo = pidinfo.split()
                if pidinfo[-1] == self.package:
                    return pidinfo[4].replace("%", '')
        return ''


class CpuMonitor(Monitor):
    def __init__(self, save_file, test_time=-1, interval=1):
        super().__init__()
        self.save_file = save_file
        self.test_time = test_time
        self.interval = interval

    def get_cpuinfo(self):
        if G.device.sdkversion >= 25:
            cpu_info = G.device.adb.raw_shell("top -n 1 -p {} -o %CPU -b -q".format(G.device.package_pid)).decode()
            cpu_info = cpu_info.strip()
        else:
            cpu_info = G.device.adb.raw_shell("top -n 1".format(G.device.package_pid)).decode()
            cpu_info = ParseCpuinfo(G.device.package, cpu_info, G.device.sdkversion).cpur_rate
        logger.info("获取到的cpu信息是：{}".format(cpu_info))
        return cpu_info

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        '''
        按照指定频率，循环搜集cpu的信息
        :return:
        '''
        cpu_title = ["timestamp", "cpu%"]
        cpu_file = self.save_file
        with open(cpu_file, 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(cpu_title)
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            cpu_list = []
            try:
                logger.debug("---------------开始获取cpu信息, into _collect_package_cpu_thread loop thread is : " + str(
                    threading.current_thread().name))
                before = time.time()
                cpu_list.append(before)
                # 为了cpu值的准确性，将采集的时间间隔放在top命令中了
                cpu_info = self.get_cpuinfo()
                cpu_list.append(cpu_info)
                after = time.time()
                time_consume = after - before
                logger.debug(
                    "============== time consume for cpu info : {0}, value {1}".format(time_consume, cpu_info))
                if cpu_info == None or cpu_info == '' or float(cpu_info) == 0:
                    logger.error("can't get cpu info")
                    G.device.get_pid()
                    logger.info("重新获取pid,重启logcat")
                    # 取消获取不到跳过，默认给0
                    cpu_list[-1] = 0
                with open(cpu_file, 'a+', encoding="utf-8") as df:
                    csv.writer(df, lineterminator='\n').writerow(cpu_list)
                    del cpu_list[:]
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in cpu thread , reason unkown!, e:")
                traceback.print_exc()
                G.device.get_pid()
        logger.debug("cpu stop event is set or timeout")


if __name__ == "__main__":
    from device import AndroidDevice

    G.device = AndroidDevice(serialno="emulator-5554", server_addr=["localhost", "5037"],
                             package="com.road7.ddtdmxandroid.ld", save_dir="localhost")
    G.device.start_app()
    cpu = CpuMonitor("./cpu.txt")
    cpu.start()
    time.sleep(15)
    cpu.stop()



# _*_ coding: utf-8 _*_
import time

from performancetest.core.cpu import CpuMonitor
from performancetest.core.device import AndroidDevice
from performancetest.core.devicebattery import DeviceBatteryMonitor
from performancetest.core.fps import FPSMonitor
from performancetest.core.global_data import GlobalData as G
from performancetest.core.gpu import GpuMonitor
from performancetest.core.logcat import Logcat
from performancetest.core.memory import MemoryMonitor


def start():
    G.device = AndroidDevice(serialno="emulator-5554", server_addr=["localhost", "5037"],
                             package="com.road7.ddtdmxandroid.ld", save_dir="localhost")
    G.logcat = Logcat(package="com.road7.ddtdmxandroid.ld", save_dir="../core/")
    time.sleep(1)
    G.device.start_app()
    CpuMonitor("./cpu.txt").start()
    MemoryMonitor("./memory.txt").start()
    FPSMonitor("./FPS.txt").start()
    GpuMonitor("./gpu.txt").start()
    DeviceBatteryMonitor("./deviceBattery.txt").start()


def stop():
    G.stop_event.clear()


if __name__ == '__main__':
    start()
    time.sleep(30)
    stop()



from abc import ABCMeta, abstractmethod
from builtins import *
from threading import Thread


class Monitor(Thread, metaclass=ABCMeta):
    def __init__(self):
        super(Monitor, self).__init__()

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def stop(self):
        pass



# coding=utf-8
import csv
import os
import platform
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger

_lock = threading.Lock()


class ParsFPSInfo(object):
    FPS_queue: list = []  # FPS collect, 每个帧对象的结果都汇集在这里，这里每次只留最后一个不完整的帧
    before_time: list = []  # 存第一秒前3帧的
    PHONE_REAL_TIME_INTERVAL: int = None  # 真实时间和手机时间的相对差值
    start_collect_time: float = None  # 开始时间测试的真实时间

    def __init__(self, surface_info):
        with _lock:
            logger.info("star instance {0}".format(ParsFPSInfo.FPS_queue))
            self.lag_number = 0  # 每秒小于24帧的次数
            self.FTIMEGE100 = 0  # 增量耗时
            self.surface_info = surface_info
            self.jank_number = [0]
            self.big_jank_number = [0]
            self.front_FPS_list = []  # 第一个完整的秒的 FPS
            self.FPS_res_info_dict: dict = {}  # 真正返回的FPS结果集 {时间：帧信息, 时间+1s: 帧信息}
            self.FPS = self.get_FPS()
            logger.info("end instance {0}".format(ParsFPSInfo.FPS_queue))

    def get_FPS(self):
        surface_info_resource = self.surface_info.split(os.linesep)
        new_FPS: tuple = self._GetSurfaceFlingerFrameData(surface_info_resource)
        self.add_new_FPS(new_FPS[1])
        logger.info("end get new fps")
        self.get_front_FPS()
        logger.info("log get fps result")
        logger.info(ParsFPSInfo.FPS_queue)
        full_FPS_number = int(1 / new_FPS[0])
        self.full_FPS_number = full_FPS_number
        # return min(full_FPS_number, len(self.front_FPS_list))
        return self.FPS_res_info_dict

    def add_new_FPS(self, new_FPS):
        if ParsFPSInfo.FPS_queue:
            FPS_list_last = ParsFPSInfo.FPS_queue[-1]
            for index, v in enumerate(new_FPS):
                if v == FPS_list_last:
                    self.FPS_queue.extend(new_FPS[index + 1:])
                    logger.info("find result")
                    logger.info("剩余帧{0}".format(len(ParsFPSInfo.FPS_queue)))
                    break
            else:
                if new_FPS and FPS_list_last < new_FPS[0]:
                    ParsFPSInfo.FPS_queue.extend(new_FPS)
                else:
                    logger.error("丢帧！！！！！！！！！！！！！！！！！！！！！！！！！！！！")
                    logger.error(FPS_list_last)
                    logger.error(new_FPS)
            if not ParsFPSInfo.PHONE_REAL_TIME_INTERVAL:
                if len(new_FPS) > 126 and new_FPS[-1] != 0.0:
                    ParsFPSInfo.PHONE_REAL_TIME_INTERVAL = int(time.time()) - new_FPS[-1]
                    logger.info("时间间隔-new-{0}, {1}".format(ParsFPSInfo.PHONE_REAL_TIME_INTERVAL, new_FPS[-1]))
        else:
            ParsFPSInfo.FPS_queue.extend(new_FPS)
            if not ParsFPSInfo.PHONE_REAL_TIME_INTERVAL:
                first_time = 0.0  # 拿到第一个有值的数据和真实时间对应
                for i in ParsFPSInfo.FPS_queue:
                    if float(i) != 0.0 and first_time == 0.0:
                        first_time = i
                logger.info("first_time {0}".format(first_time))
                if first_time != 0.0:
                    if ParsFPSInfo.start_collect_time:
                        ParsFPSInfo.PHONE_REAL_TIME_INTERVAL = ParsFPSInfo.start_collect_time - int(first_time)
                        # ParsFPSInfo.start_collect_time = None, 不会中断开始收集时间有一个就够了
                    else:
                        ParsFPSInfo.PHONE_REAL_TIME_INTERVAL = int(time.time()) - int(first_time)
                    logger.info(
                        "时间间隔 {0} {1} {2} {3}".format(ParsFPSInfo.PHONE_REAL_TIME_INTERVAL, time.time(), first_time,
                                                      ParsFPSInfo.FPS_queue))
                else:
                    ParsFPSInfo.FPS_queue = []
                    ParsFPSInfo.PHONE_REAL_TIME_INTERVAL = None
                    logger.info("第一个时间戳有误")
                    logger.info(ParsFPSInfo.FPS_queue)

    def get_front_FPS(self):
        """
        PerfDog Jank计算方法：
        同时满足两条件，则认为是一次卡顿Jank.
        ①Display FrameTime>前三帧平均耗时2倍。
        ②Display FrameTime>两帧电影帧耗时 (1000ms/24*2=84ms)。
        同时满足两条件，则认为是一次严重卡顿BigJank.
        ①Display FrameTime >前三帧平均耗时2倍。
        ②Display FrameTime >三帧电影帧耗时(1000ms/24*3=125ms)。
        """
        # 拿到了队列里所有的完整帧
        while ParsFPSInfo.FPS_queue:
            logger.info("join get front fps")
            tmp = []
            time_flag = ParsFPSInfo.FPS_queue.pop(0)
            time_second = int(time_flag)
            tmp.append(time_flag)
            while ParsFPSInfo.FPS_queue and int(ParsFPSInfo.FPS_queue[0]) == time_second:
                header_ele = ParsFPSInfo.FPS_queue.pop(0)
                tmp.append(header_ele)
            if not ParsFPSInfo.FPS_queue:
                ParsFPSInfo.FPS_queue.extend(tmp)
                break
            else:
                self.front_FPS_list.append(tmp)
        res_dict = {}
        try:
            for item_list in self.front_FPS_list:
                # print(item_list[0], ParsFPSInfo.PHONE_REAL_TIME_INTERVAL)
                # 如果当前帧都是空就跳过
                if sum(item_list) == 0.0:
                    continue
                first_time_head = None  # 获取第一个帧第一个有时间的值
                for time_head in item_list:
                    first_time_head = time_head if time_head else None
                res_dict[int(first_time_head) + ParsFPSInfo.PHONE_REAL_TIME_INTERVAL] = item_list[0:]
        except Exception as e:
            logger.error(e)
            traceback.print_exc()
        self.FPS_res_info_dict: dict = res_dict
        self.get_jank(self.FPS_res_info_dict)

    def get_jank(self, FPS_res_info_dict: dict):
        for front_index, (time_number, item_list_v) in enumerate(FPS_res_info_dict.items()):
            for index, v in enumerate(item_list_v):
                if len(ParsFPSInfo.before_time) < 4:
                    ParsFPSInfo.before_time.append(v)
                else:
                    interval = v - ParsFPSInfo.before_time[-1]
                    if interval > 0.1:
                        ParsFPSInfo.FTIMEGE100 = 1
                    else:
                        ParsFPSInfo.FTIMEGE100 = 0
                    if v - ParsFPSInfo.before_time[-1] > sum([
                        ParsFPSInfo.before_time[-1] - ParsFPSInfo.before_time[-2],
                        ParsFPSInfo.before_time[-2] - ParsFPSInfo.before_time[-3],
                        ParsFPSInfo.before_time[-3] - ParsFPSInfo.before_time[-4], ]) / 3 * 2:
                        if interval > 0.125:
                            if len(self.big_jank_number) <= front_index:
                                self.big_jank_number.append(1)
                            else:
                                self.big_jank_number[front_index] += 1
                        elif interval > 0.084:
                            if len(self.jank_number) <= front_index:
                                self.jank_number.append(1)
                            else:
                                self.jank_number[front_index] += 1
                    ParsFPSInfo.before_time.pop(0)
                    ParsFPSInfo.before_time.append(v)

    def _GetSurfaceFlingerFrameData(self, results):
        """Returns collected SurfaceFlinger frame timing data.

        Returns:
          A tuple containing:
          - The display's nominal refresh period in seconds.
          - A list of timestamps signifying frame presentation times in seconds.
          The return value may be (None, None) if there was no data collected (for
          example, if the app was closed before the collector thread has finished).
        """
        # adb shell dumpsys SurfaceFlinger --latency <window name>
        # prints some information about the last 128 frames displayed in
        # that window.
        # The data returned looks like this:
        # 16954612
        # 7657467895508   7657482691352   7657493499756
        # 7657484466553   7657499645964   7657511077881
        # 7657500793457   7657516600576   7657527404785
        # (...)
        #
        # The first line is the refresh period (here 16.95 ms), it is followed
        # by 128 lines w/ 3 timestamps in nanosecond each:
        # A) when the app started to draw
        # B) the vsync immediately preceding SF submitting the frame to the h/w
        # C) timestamp immediately after SF submitted that frame to the h/w
        #
        # The difference between the 1st and 3rd timestamp is the frame-latency.
        # An interesting data is when the frame latency crosses a refresh period
        # boundary, this can be calculated this way:
        #
        # ceil((C - A) / refresh-period)
        #
        # (each time the number above changes, we have a "jank").
        # If this happens a lot during an animation, the animation appears
        # janky, even if it runs at 60 fps in average.
        #
        # We use the special "SurfaceView" window name because the statistics for
        # the activity's main window are not updated when the main web content is
        # composited into a SurfaceView.
        timestamps = []
        if not len(results):
            return (None, None)

        # 针对oppo Find 2x 的一款手机兼容
        if "TIME" in results[0]:
            results = results[1:]

        nanoseconds_per_second = 1e9
        refresh_period = int(results[0]) / nanoseconds_per_second

        # If a fence associated with a frame is still pending when we query the
        # latency data, SurfaceFlinger gives the frame a timestamp of INT64_MAX.
        # Since we only care about completed frames, we will ignore any timestamps
        # with this value.
        pending_fence_timestamp = (1 << 63) - 1

        for index, line in enumerate(results[1:]):
            fields = line.split()
            if len(fields) != 3:
                continue
            try:
                timestamp = int(fields[1])
            except ValueError as e:
                logger.error(e)
                continue
            if timestamp == pending_fence_timestamp:
                continue
            timestamp /= nanoseconds_per_second
            # if index == 0:
            #     for _, v in enumerate(timestamps):
            #         if v == timestamp:
            #             timestamps = timestamps[:_]
            #             break
            timestamps.append(timestamp)
        return (refresh_period, timestamps)


class FPSMonitor(Monitor):
    def __init__(self, save_file, interval=1, test_time=-1):
        super().__init__()
        self.save_file = save_file
        self._stop_event = threading.Event()
        self.interval = interval
        self.m_surface_view_name = None

    def get_FPS_info(self):
        """
        通过执行 "dumpsys SurfaceFlinger --list" 命令，您可以获取有关当前正在显示在屏幕上的所有图形元素的详细信息，例如它们的名称、大小、格式、位置和可见性等。
        """
        surface_list_info = G.device.adb.raw_shell("dumpsys SurfaceFlinger --list").decode()
        if G.device.sdkversion >= 24:
            self.get_FPS_surface_view(surface_list_info)
        else:
            self.get_FPS_surface_view_low_version(surface_list_info)
        """
        通过执行 "dumpsys SurfaceFlinger --latency" 命令，您可以获取有关SurfaceFlinger处理每一帧图像所需的时间的信息，包括处理时间的最小值、最大值、平均值和标准差等。
        """
        surface_info = G.device.adb.raw_shell(
            "dumpsys SurfaceFlinger --latency '{0}' ".format(self.m_surface_view_name)).decode()
        pars_FPS_info = ParsFPSInfo(surface_info)
        self.FPS_info = pars_FPS_info
        return pars_FPS_info.FPS, pars_FPS_info.lag_number, pars_FPS_info.full_FPS_number,

    def get_FPS_surface_view(self, surface_info):
        for i in surface_info.split(os.linesep):
            if "SurfaceView" not in i:
                continue
            if "Background for" in i:
                continue
            if G.device.package not in i:
                continue
            if not self.m_surface_view_name or "BLAST" in i:
                self.m_surface_view_name = i
        if self.m_surface_view_name is None:
            self.m_surface_view_name = self.get_surfaceview_activity()
        logger.info("获取到的surface_view信息是：{}".format(self.m_surface_view_name))

    def get_FPS_surface_view_low_version(self, surface_info):
        for i in surface_info.split(os.linesep):
            if G.device.package not in i:
                continue
            if not self.m_surface_view_name or "BLAST" in i:
                self.m_surface_view_name = i.strip()
        logger.info("获取到的surface_view信息是：{}".format(self.m_surface_view_name))

    def get_surfaceview_activity(self):
        activity_name = ''
        activity_line = ''
        try:
            dumpsys_result = G.device.adb.raw_shell('dumpsys SurfaceFlinger --list | {} {}'.format(('grep', 'findstr')[platform.system() == "Windows"], G.device.package))
            dumpsys_result_list = dumpsys_result.split('\n')
            for line in dumpsys_result_list:
                if line.startswith('SurfaceView') and line.find(G.device.package) != -1:
                    activity_line = line.strip()
                    break
            if activity_line:
                if activity_line.find(' ') != -1:
                    activity_name = activity_line.split(' ')[2]
                else:
                    activity_name = activity_line.replace('SurfaceView', '').replace('[', '').replace(']', '')
            else:
                activity_name = dumpsys_result_list[len(dumpsys_result_list) - 1]
                if not activity_name.__contains__(G.device.package):
                    logger.error(
                        'get activity name failed, Please provide SurfaceFlinger --list information to the author')
                    logger.info('dumpsys SurfaceFlinger --list info: {}'.format(dumpsys_result))
        except Exception:
            traceback.print_exc()
            logger.error('get activity name failed, Please provide SurfaceFlinger --list information to the author')
            logger.info('dumpsys SurfaceFlinger --list info: {}'.format(dumpsys_result))
        return activity_name

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        G.device.adb.raw_shell("dumpsys SurfaceFlinger --latency-clear").decode()
        '''
        按照指定频率，循环搜集FPS的信息
        '''
        # ftimege100:增量耗时
        FPS_title = ["timestamp", "FPS", "lag_number", "FPS_full_number", "jank_number", "big_jank_number",
                     "ftimege100"]
        FPS_file = self.save_file
        with open(FPS_file, 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(FPS_title)
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        ParsFPSInfo.start_collect_time = int(time.time())
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            FPS_list = []
            try:
                logger.debug("---------------开始获取手机fps信息, into _collect_package_FPS_thread loop thread is : " + str(
                    threading.current_thread().name))
                before = time.time()
                try:
                    FPS_info_FPS, FPS_info_lag_number, FPS_full_number = self.get_FPS_info()
                except Exception as e:
                    logger.info(e)
                logger.info(
                    "fps collect result {0} {1} {2}".format(FPS_info_FPS, FPS_info_lag_number, FPS_full_number))
                after = time.time()
                time_consume = after - before
                if not FPS_info_FPS:
                    logger.info("未获取到数据")
                    delta_inter = self.interval - time_consume
                    if delta_inter > 0:
                        time.sleep(delta_inter)
                        continue
                logger.debug("  ============== time consume for fps info : " + str(time_consume))
                for front_index, (time_item, item_value_list) in enumerate(FPS_info_FPS.items()):
                    FPS_list.append(time_item)
                    FPS_list.append(len(item_value_list) if len(item_value_list) < FPS_full_number else FPS_full_number)
                    FPS_list.append(1 if len(item_value_list) < 24 else 0)
                    FPS_list.append(FPS_full_number)
                    try:
                        FPS_list.append(
                            self.FPS_info.jank_number[front_index] if len(
                                self.FPS_info.jank_number) > front_index else 0)
                    except Exception as e:
                        FPS_list.append(0)
                        logger.info(e)
                    try:
                        FPS_list.append(self.FPS_info.big_jank_number[front_index] if len(
                            self.FPS_info.big_jank_number) > front_index else 0)
                    except Exception as e:
                        FPS_list.append(0)
                        logger.info(e)
                    try:
                        FPS_list.append(ParsFPSInfo.FTIMEGE100)
                    except Exception as e:
                        FPS_list.append(0)
                        logger.info(e)
                    logger.info(FPS_file + "*****************************")
                    with open(FPS_file, 'a+', encoding="utf-8") as df:
                        csv.writer(df, lineterminator='\n').writerow(FPS_list)
                        del FPS_list[:]
                    delta_inter = self.interval - time_consume
                    if delta_inter > 0:
                        time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in FPS thread , reason unkown!, e:")
                traceback.format_exc()
                G.device.get_pid()
        logger.debug("FPS stop event is set or timeout")



# coding=utf-8
import subprocess
import traceback
from builtins import *

import requests

from performancetest.core.global_data import logger


def split_cmd(cmds):
    return cmds.split() if isinstance(cmds, str) else list(cmds)


def proc_communicate_timeout(proc, timeout):
    try:
        stdout, stderr = proc.communicate(timeout=timeout)
    except subprocess.TimeoutExpired as e:
        proc.kill()
        stdout, stderr = proc.communicate()
        exp = Exception("Command {cmd} timed out after {timeout} seconds: stdout['{stdout}'], "
                        "stderr['{stderr}']".format(cmd=proc.args, timeout=e.timeout,
                                                    stdout=stdout, stderr=stderr))
        raise (exp)

    return stdout, stderr


class ADB(object):
    def __init__(self, serialno=None, adb_path=None, server_addr=None):
        self.serialno = serialno
        self.adb_path = adb_path or "adb"
        self._set_cmd_options(server_addr)
        self.server_addr = server_addr

    def _set_cmd_options(self, server_addr=None):
        logger.info(server_addr)
        self.host = server_addr[0] if server_addr[0] else "127.0.0.1"
        self.port = server_addr[1] if server_addr[1] else 5037
        self.cmd_options = [self.adb_path]
        if self.host:
            self.cmd_options += ['-H', self.host]
        if self.port:
            self.cmd_options += ['-P', str(self.port)]

    def start_shell(self, cmds):
        """
        Handle `adb shell` c(s)

        Args:
            cmds: adb shell command(s)

        Returns:
            None

        """
        cmds = ['shell'] + split_cmd(cmds)
        return self.start_cmd(cmds)

    def start_cmd(self, cmds, device=True, only_args=False):
        """
        Start a subprocess with adb command(s)

        Args:
            cmds: command(s) to be run
            device: if True, the device serial number must be specified by `-s serialno` argument

        Raises:
            RuntimeError: if `device` is True and serialno is not specified

        Returns:
            a subprocess

        """
        if device:
            if not self.serialno:
                raise RuntimeError("please set serialno first")
            cmd_options = self.cmd_options + ['-s', self.serialno]
        else:
            cmd_options = self.cmd_options

        cmds = cmd_options + split_cmd(cmds)
        logger.debug(cmds)
        logger.debug(" ".join(cmds))
        if only_args:
            return cmds
        proc = subprocess.Popen(
            cmds,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            creationflags=0
        )
        return proc

    def raw_shell(self, cmds, ensure_unicode=True):
        cmds = ['shell'] + split_cmd(cmds)
        logger.info("shell执行命令：{}".format(str(cmds)))
        out = self.cmd(cmds, ensure_unicode=False)
        return out

    def cmd(self, cmds, device=True, ensure_unicode=True, timeout=None):
        proc = self.start_cmd(cmds, device)
        if timeout:
            stdout, stderr = proc_communicate_timeout(proc, timeout)
        else:
            stdout, stderr = proc.communicate()
        logger.error(f"err msg {stderr}")
        return stdout

    def killpid(self, pid):
        logger.info("进入killpid函数")
        self.raw_shell("kill {}".format(pid))
        logger.info("killpid函数执行结束")



# coding=utf-8
import os
import subprocess
import threading
import time
import traceback
from builtins import *
from io import BytesIO

import tidevice
from PIL import Image
from func_timeout import func_set_timeout
from tidevice import DataType
from tidevice._perf import RunningProcess, iter_cpu_memory, iter_gpu, iter_fps

from performancetest.core.command import ADB
from performancetest.core.global_data import logger


class AndroidDevice(object):
    def __init__(self, serialno, server_addr, package, save_dir):
        self.adb = ADB(serialno=serialno, server_addr=server_addr)
        self.serialno = serialno
        self.server_addr = server_addr
        self.package_pid = ''
        self.package = package
        self.save_dir = save_dir
        self.collect_logcat = threading.Thread(target=self._collect_logcat)
        self.sdkversion = self.get_sdkversion()

    def get_sdkversion(self):
        sdkversion = self.adb.raw_shell("getprop ro.build.version.sdk")
        logger.info("获取到的sdk版本号是{}".format(sdkversion))
        sdkversion = sdkversion.decode().strip()
        if sdkversion:
            return int(sdkversion)
        else:
            # 没有获取到sdkverison默认返回25
            return 25

    def get_front_app(self):
        lines: str = self.adb.raw_shell("dumpsys activity top | grep ACTIVITY").decode()
        logger.info("获取到的activty是{0}".format(lines))
        return lines.strip()

    def app_is_front(self):
        lines = self.get_front_app()
        lines = lines.split(os.linesep)
        for line in lines:
            if self.package in line:
                break
        else:
            raise Exception("app未启动")
        return (self.package in lines[-1]) and ("WebLandActivity" not in lines[-1]) and (
                "ali" not in lines[-1].lower()) and ("wx" not in lines[-1].lower())

    def get_packgelist(self):
        stdout = self.adb.raw_shell("pm list package")
        return stdout

    def is_install_package(self, package):
        stdout = self.adb.raw_shell("pm list package")
        logger.info("pm list package 返回内容：{0}".format(str(stdout)))
        if stdout.decode().find("package:" + package + os.linesep) > -1:
            return True
        else:
            return False

    def start_collect_logcat(self):
        self.collect_logcat.start()

    def _collect_logcat(self):
        with open(os.path.join(self.save_dir, "logcat.txt"), "w+") as f:
            pid = self.get_pid()
            collect_cmd = "adb logcat --pid=={}".format(pid)
            subprocess.run(collect_cmd.split(" "), stdout=f, stderr=f)

    def install_apk(self, apk_route):
        self.adb.raw_shell(["adb", "install", apk_route])

    @func_set_timeout(5)
    def stop_app(self):
        self.adb.raw_shell(["am", "force-stop", self.package])

    def start_app(self):  # 启动app
        self.adb.raw_shell(['monkey', '-p', self.package, '-c', 'android.intent.category.LAUNCHER', '1'])

    def get_pid(self):
        try:
            pid_infos = self.adb.raw_shell("ps | grep " + self.package).decode()
            real_pid = None
            pid_infos = pid_infos.splitlines()
            for pid in pid_infos:
                if pid.split()[-1] == self.package:
                    real_pid = pid.split()[1]
            if not real_pid:
                real_pid = pid_infos.split()[1]
            logger.info("测试包获取到的pid是{}".format(real_pid))
            self.package_pid = real_pid
            return real_pid
        except IndexError as e:
            logger.error("获取到的pid信息是{}".format(pid_infos))
            logger.exception(e)


class IosDevice(object):
    def __init__(self, serialno, device_addr, package, save_dir):
        self.serialno = serialno
        self.device = tidevice.Device(udid=self.serialno)
        self.device_addr = device_addr
        self.package = package
        self.save_dir = save_dir
        self.perf_dict_iter = dict()

    def start_app(self):  # 启动app
        self.device.app_start(self.package)

    def stop_app(self):
        self.device.app_stop(self.package)

    def get_battery(self):
        res = self.device.get_io_power()
        res_text = {"CurrentCapacity": (res['Diagnostics']['IORegistry']['CurrentCapacity']), "Temperature": (
            res['Diagnostics']['IORegistry']['Temperature'])}
        return res_text

    def get_perf(self):
        if not self.perf_dict_iter.get(self.package):
            perf = tidevice.Performance(self.device, [DataType.CPU, DataType.MEMORY, DataType.FPS, DataType.GPU])
            _rp = RunningProcess(self.device, self.package)
            perf._rp = _rp
            self.perf_dict_iter[self.package] = {"cm": iter_cpu_memory(self.device, _rp), "fps": iter_fps(self.device),
                                                 "gpu": iter_gpu(self.device)}
        perf_info = None
        try:
            perf_info = self.gen_perf_info(self.package)
        except:
            traceback.print_exc()
            del self.perf_dict_iter[self.package]
        return perf_info

    def gen_perf_info(self, key) -> list:
        cur_list: list = []
        cur_time = int(time.time() * 1000)
        dict_perf = self.perf_dict_iter[key]
        c_m = dict_perf.get("cm")
        fps = dict_perf.get("fps")
        gpu = dict_perf.get("gpu")
        c_type, c_value = next(c_m)
        c_value["timestamp"] = cur_time
        cur_list.append({"value": c_value, "type": c_type.lower()})
        c_type, c_value = next(c_m)
        c_value["timestamp"] = cur_time
        cur_list.append({"value": c_value, "type": c_type.lower()})
        c_type, c_value = next(fps)
        c_value["timestamp"] = cur_time
        cur_list.append({"value": c_value, "type": c_type.lower()})
        c_type, c_value = next(gpu)
        c_value["timestamp"] = cur_time
        cur_list.append({"value": c_value, "type": c_type.lower()})
        return cur_list

    def screenshot(self, filename):
        screenshot_image = self.device.screenshot()
        byte_stream = BytesIO()
        screenshot_image.save(byte_stream, format='PNG', quality_api=30)
        if byte_stream:
            image = Image.open(byte_stream)
            image.save(filename)



# _*_ coding: utf-8 _*_

from builtins import *

from pydantic import BaseModel


class TaskEntity(BaseModel):
    serialno: str
    port: int
    package: str
    platform: str  # android | ios


class DeviceEntity(BaseModel):
    serialno: str
    platform: str  # android | ios


class PackageEntity(BaseModel):
    serialno: str
    package: str
    platform: str  # android | ios



# # _*_ coding: utf-8 _*_
from setuptools import setup, find_packages

setup(
       name='performancetest',
       version='0.0.15',
       url='https://github.com/1033866383/perf-orange-cat',
       author='bozhou.fan',
       author_email='15525730080@163.com',
       description='Android, IOS app_performance',
       packages=find_packages(),
       install_requires=[
           "psutil", "airtest", "fastapi", "tidevice", "func-timeout", "sqlalchemy", "sqlalchemy-serializer", "uvicorn"
       ],
       include_package_data=True,  # 这里添加 include_package_data 参数
       package_data={
           'performancetest': ['web/test_result/*']
       },
       classifiers=[
           'Development Status :: 3 - Alpha',
           'Intended Audience :: Developers',
           'License :: OSI Approved :: MIT License',
           'Programming Language :: Python :: 3',
           'Programming Language :: Python :: 3.9',
       ],
   )


import concurrent.futures
import os
import traceback
from builtins import *

import numpy as np

from performancetest.core.global_data import logger


class DataCollect(object):

    def __init__(self, file_dir_path=None):
        """
        任务所在文件夹, 设置了这个文件夹其他的不需要再传file_path
        """
        self.file_dir_path = file_dir_path
        self.is_need_relative_time = False

    def __read_csv_file(self, file_path=None, skip_rows=1, usecols=None):
        """
        读取cpu，memory，fps，gpu，温度等csv文件的值
        """
        if not usecols:
            csv_data = np.genfromtxt(file_path, skip_header=skip_rows, delimiter=",", dtype=float, filling_values=0)
        else:
            csv_data = np.genfromtxt(file_path, skip_header=skip_rows, delimiter=",", dtype=float, filling_values=0,
                                     usecols=usecols)
        return csv_data

    def __read_cpu(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "cpu.csv")
        return self.__read_csv_file(file_path=file_path)

    def __read_memory(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "memory.csv")
        return self.__read_csv_file(file_path=file_path)

    def __read_gpu(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "gpu.csv")
        return self.__read_csv_file(file_path=file_path)

    def __read_device_temperature(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "devicebattery.csv")
        return self.__read_csv_file(file_path=file_path, usecols=(0, 1))

    def __read_device_battery_level(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "devicebattery.csv")
        return self.__read_csv_file(file_path=file_path, usecols=(0, 2))

    def __read_fps(self, file_path=None):
        if not file_path:
            file_path = os.path.join(self.file_dir_path, "fps.csv")
        return self.__read_csv_file(file_path=file_path)

    # 监控类型对应的读取方法
    __monitortype_func = {"cpu": __read_cpu, "memory": __read_memory, "fps": __read_fps,
                          "gpu": __read_gpu, "devicebatterytemperature": __read_device_temperature,
                          "devicebatterylevel": __read_device_battery_level}

    # 读数据的方法使用：DataCollect.read_data(1, "cpu", "memory", "fps")
    @classmethod
    def read_data(cls, file_dir_path: int, /, *args, **kwargs):
        """
        param: "cpu", "memory"
        return:
        {
            public_imgs：[{"16xxxxxx": "pic_path"}, {"16xxxxxx": "pic_path1"}]
            public_start_time：16xxxxxx: int,
            public_end_time：16xxxxxx: int,
            cpu: {
                 time：[16xxxxxx, 16xxxxxx]  //cpu,memory,fps 开始真实时间戳相同
                 value: [100, 101]
                 relative_time: [00:00, 00:10]
            },
            memory: {
                  time：[16xxxxxx, 16xxxxxx]  //cpu,memory,fps 开始真实时间戳相同
                 value: [100, 101]

            }
        }
        """
        return cls.item_result(file_dir_path, monitortypes=args,
                               is_need_relative_time=kwargs.get("is_need_relative_time", True))

    # 读所有类型的数据的方法使用：DataCollect.read_data(1, "cpu", "memory", "fps")
    @classmethod
    def read_data_all(cls, file_dir_path):
        return cls.read_data(file_dir_path, *cls.__monitortype_func.keys())

    @classmethod
    def item_result(cls, file_dir_path: int, monitortypes: tuple, **kwargs):
        data_collect = DataCollect(file_dir_path=file_dir_path)
        result_dict: dict = {}  # 存储每种监控类型的结果
        with concurrent.futures.ThreadPoolExecutor() as executor:
            # 执行所有读取
            wait_task: list = []
            for monitor_name in monitortypes:
                future = executor.submit(cls.__monitortype_func.get(monitor_name), data_collect)
                result_dict[monitor_name] = future
                wait_task.append(future)
            # wait task end
            done, not_done = concurrent.futures.wait(wait_task, return_when=concurrent.futures.ALL_COMPLETED)
            for key, value_future in list(result_dict.items()):
                try:
                    value_future_res = value_future.result()
                    if len(value_future_res.shape) < 2:
                        raise Exception("数组低于2维度")
                    if value_future_res.size <= 0:
                        raise Exception("数据结果为空")
                    result_dict[key] = value_future_res
                    # 如果是fps 会去掉第一个和 最后一个点
                    if key == "fps":
                        result_dict[key] = result_dict[key][1: -1] if len(
                            result_dict[key]) > 3 else result_dict[key]
                except Exception as e:
                    logger.error(e)
                    result_dict.pop(key, None)

            # 处理公共开始时间, 结束时间
            # logger.info("item{0}".format(result_dict))
            if result_dict:
                public_start_time, public_end_time = cls.get_public_time(result_dict)
                # 获取所有结果
                for (monitor_name, future_result) in result_dict.items():
                    result_dict[monitor_name] = cls.time_value_result(monitor_name, future_result,
                                                                      public_start_time, public_end_time,
                                                                      is_need_relative_time=kwargs.get(
                                                                          "is_need_relative_time", False))
                result_dict["public_start_time"] = public_start_time
                result_dict["public_end_time"] = public_end_time
                img_path_dir = os.path.join(file_dir_path, "picture_log")
                if os.path.exists(img_path_dir):
                    task_dir, task_name_int = os.path.split(file_dir_path)
                    _, host = os.path.split(task_dir)
                    result_dict["public_imgs"] = cls.get_public_imgs(img_path_dir, public_start_time,
                                                                     public_end_time, task_name_int, host)
        return result_dict

    @staticmethod
    def get_public_imgs(img_path_dir: str, public_start_time: int, public_end_time: int, task_name_int: str, host: str):
        all_imgs = os.listdir(img_path_dir)
        img_time_dict = {i: "" for i in range(public_start_time, public_end_time + 1)}
        for img in all_imgs:
            try:
                img_time_dict[int(int(
                    img.replace(".jpg", "")) * 0.001)] = "/static/{0}/{1}/picture_log/{2}".format(host, task_name_int,
                                                                                                  img)
            except Exception as e:
                logger.error(e)
                traceback.print_exc()
                continue
        res_list = []
        for key, v in img_time_dict.items():
            try:
                res_list.append({"time": key, "picture_path": v,
                                 "relative_time": DataCollect.seconds_to_time(int(key) - public_start_time)})
            except Exception as e:
                logger.error(e)
        res_list.sort(key=lambda x: int(x.get("time")))
        return res_list

    @staticmethod
    def get_public_time(result_dict: dict):
        time_collect: list[list] = [list(map(lambda x: int(x), future_result[:, 0])) for monitor_name, future_result in
                                    result_dict.items()]  # 所有的时间[[], []]
        public_start_time, public_end_time = DataCollect.find_common_elements(time_collect)
        return public_start_time, public_end_time

    @staticmethod
    def find_common_elements(lists):

        min_time = None
        max_time = None
        for item_list in lists:
            if not min_time:
                min_time = item_list[0]
            if not max_time:
                max_time = item_list[-1]
            max_time = max(max_time, max(item_list))
            min_time = min(min_time, min(item_list))
        return min_time, max_time

    # 获取不同类型的数据，掐头去尾保证所有的数据起点终点一致
    @staticmethod
    def time_value_result(monitor_name, csv_data, start_time, end_time, **kwargs):
        real_time: list = csv_data[:, 0].tolist()
        value: list = np.round(csv_data[:, 1], 2).tolist()
        value_max = max(value)
        value_min = min(value)
        value_avg = sum(value) / len(value) if value and len(value) else 0
        real_time_int: list = list(map(lambda x: int(x), real_time))
        head_lack_second = real_time_int[0] - start_time
        end_lack_second = end_time - real_time_int[-1]
        head_time = [real_time_int[0] + i for i in range(head_lack_second)]
        end_time = [real_time_int[-1] + i for i in range(end_lack_second)]
        head_time_value = ["-" for i in range(head_lack_second)]
        end_time_value = ["-" for i in range(end_lack_second)]
        res_dict = {"time": head_time + real_time + end_time, "value": head_time_value + value + end_time_value,
                    "max": value_max, "min": value_min, "avg": value_avg}
        if kwargs.get("is_need_relative_time", False):
            res_dict["relative_time"] = [DataCollect.seconds_to_time(item - start_time) for item in
                                         head_time + real_time_int + end_time]
        if monitor_name == "fps":
            try:
                res_dict["full_number"] = max(csv_data[:, 3])  # 满帧
                source_jank_number = csv_data[:, 4].tolist()
                source_big_jank_number = csv_data[:, 5].tolist()
                res_dict["jank_number_sum"] = sum(source_jank_number)
                res_dict["big_jank_number_sum"] = sum(source_big_jank_number)
                res_dict["all_jank_rate"] = (sum(source_jank_number) + sum(source_big_jank_number)) / len(
                    res_dict["time"]) * 100
                res_dict["jank_number"] = head_time_value + source_jank_number + end_time_value  # 卡顿
                res_dict["big_jank_number"] = head_time_value + source_big_jank_number + end_time_value  # 强卡顿
                res_dict["ftimege100"] = head_time_value + csv_data[:, 6].tolist() + end_time_value  # 增量耗时
            except Exception as e:
                res_dict["full_number"] = 0
                res_dict["jank_number"] = []
                res_dict["big_jank_number"] = []
                res_dict["ftimege100"] = []
                res_dict["jank_number_sum"] = 0
                res_dict["big_jank_number_sum"] = 0
                res_dict["all_jank_rate"] = 0
                logger.error(e)
            # fps值需要去掉开头一个和最后一个
            # res_dict["time"] = res_dict["time"][1: -2] if res_dict.get("time") else []
            # res_dict["value"] = res_dict["value"][1: -2] if res_dict.get("value") else []
        return res_dict

    @staticmethod
    def seconds_to_time(time_data_collect):
        minutes = str(time_data_collect // 60).zfill(2)
        seconds = str(time_data_collect % 60).zfill(2)
        return f"{minutes}:{seconds}"



# coding=utf-8
import csv
import re
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger


class ParsMeminfo(object):
    RE_TOTAL_PSS = re.compile(r'TOTAL\s+(\d+)')

    def __init__(self, meminfo):
        self.meminfo = meminfo
        self.taltol_pss = self.get_taltol_pss()

    def get_taltol_pss(self):
        match = self.RE_TOTAL_PSS.search(self.meminfo)
        if match:
            return round(float(match.group(1)) / 1024, 2)
        else:
            return ""


class MemoryMonitor(Monitor):
    def __init__(self, save_file, interval=1, test_time=-1):
        super().__init__()
        self.save_file = save_file
        self.test_time = test_time
        self.interval = interval

    def get_mem_info(self):
        if G.device.sdkversion >= 25:
            mem_info = G.device.adb.raw_shell("top -n 1 -p {} -o RES -b -q".format(G.device.package_pid)).decode()
            mem_info = mem_info.strip()
            return mem_info
        else:
            mem_info = G.device.adb.raw_shell("dumpsys meminfo {}".format(G.device.package)).decode()
            mem_info = str(ParsMeminfo(mem_info).taltol_pss)
        logger.info("当前获取到的mem信息是{}".format(mem_info))
        return mem_info

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        '''
        按照指定频率，循环搜集内存的信息
        '''
        mem_title = ["timestamp", "memory"]
        mem_file = self.save_file
        with open(mem_file, 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(mem_title)
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            mem_list = []
            try:
                logger.debug("---------------开始获取手机内存信息, into _collect_package_mem_thread loop thread is : " + str(
                    threading.current_thread().name))
                before = time.time()
                mem_list.append(before)
                # 为了mem值的准确性，将采集的时间间隔放在top命令中了
                mem_info = self.get_mem_info()
                if "G" in mem_info:
                    mem_info = float(mem_info.replace("G", "")) * 1024
                elif "M" in mem_info:
                    mem_info = float(mem_info.replace("M", ""))
                mem_list.append(mem_info)
                after = time.time()
                time_consume = after - before
                logger.debug("  ============== time consume for mem info : " + str(time_consume))
                if mem_list == None or mem_list == '':
                    logger.debug("can't get memory info")
                    G.device.get_pid()
                logger.info(mem_file + "*****************************")
                if not mem_info:
                    mem_list[-1] = 0
                with open(mem_file, 'a+', encoding="utf-8") as df:
                    csv.writer(df, lineterminator='\n').writerow(mem_list)
                    del mem_list[:]
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in memory thread , reason unkown!, e:")
                logger.error(traceback.format_exc())
                G.device.get_pid()
        logger.debug("memory stop event is set or timeout")



from abc import ABCMeta, abstractmethod
from builtins import *


class Actuator(metaclass=ABCMeta):
    def __init__(self):
        super(Actuator, self).__init__()

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def stop(self):
        pass



# coding=utf-8

import csv
import os
import threading
import time
import traceback
from builtins import *

from performancetest.core.base.monitor import Monitor
from performancetest.core.global_data import GlobalData as G, logger


class ParseGpuinfo(object):
    def __init__(self, package, gpuinfo, sdkversion=None):
        self.gpuinfo = gpuinfo
        self.package = package
        self.sdkversion = sdkversion
        self.gpur_rate = self.get_gpu_rate()

    def get_gpu_rate(self):
        for pidinfo in self.gpuinfo.split(os.linesep):
            if self.package in pidinfo:
                pidinfo = pidinfo.split()
                if pidinfo[-1] == self.package:
                    return pidinfo[4].replace("%", '')
        return ''


class GpuMonitor(Monitor):
    def __init__(self, save_file, test_time=-1, interval=1):
        super().__init__()
        self.save_file = save_file
        self.test_time = test_time
        self._stop_event = threading.Event()
        self.interval = interval

    def get_gpuinfo(self):
        gpu_info = G.device.adb.raw_shell("cat /sys/class/kgsl/kgsl-3d0/gpubusy").decode()
        gpu_info = gpu_info.strip()
        res_n = gpu_info.split(" ")
        for i in range(len(res_n) - 1, -1, -1):
            if res_n[i] == '':
                res_n.pop(i)
        gpu_info = 0
        try:
            gpu_info = int(res_n[0]) / int(res_n[1]) * 100
        except:
            pass
        logger.info("获取到的gpu信息是：{}".format(gpu_info))
        return gpu_info

    def start(self):
        super(Monitor, self).start()

    # 结束任务
    def stop(self):
        G.stop_event.clear()

    # 暂停任务
    def suspend(self):
        G.suspend_event.clear()

    def run(self):
        '''
        按照指定频率，循环搜集gpu的信息
        :return:
        '''
        gpu_title = ["timestamp", "gpu%"]
        gpu_file = self.save_file
        with open(gpu_file, 'w+') as df:
            csv.writer(df, lineterminator='\n').writerow(gpu_title)
        G.stop_event.set()  # 启动
        G.suspend_event.set()  # 启动
        while G.stop_event.is_set():  # 停止了循环会停止
            G.suspend_event.wait()  # 暂停时会暂停在这里
            gpu_list = []
            try:
                logger.debug("---------------开始获取gpu信息, into _collect_package_gpu_thread loop thread is : " + str(
                    threading.current_thread().name))
                before = time.time()
                gpu_list.append(before)
                # 为了gpu值的准确性，将采集的时间间隔放在top命令中了
                gpu_info = self.get_gpuinfo()
                gpu_list.append(gpu_info)
                after = time.time()
                time_consume = after - before
                logger.debug("  ============== time consume for gpu info : " + str(time_consume))
                if gpu_info == None or gpu_info == '' or float(gpu_info) == 0:
                    logger.error("can't get gpu info")
                    G.device.get_pid()
                    # 取消获取不到跳过，默认给0
                    gpu_list[-1] = 0
                with open(gpu_file, 'a+', encoding="utf-8") as df:
                    csv.writer(df, lineterminator='\n').writerow(gpu_list)
                    del gpu_list[:]
                delta_inter = self.interval - time_consume
                if delta_inter > 0:
                    time.sleep(delta_inter)
            except Exception as e:
                logger.error("an exception hanpend in gpu thread , reason unkown!, e:")
                logger.error(e)
                traceback.print_exc()
                G.device.get_pid()
        logger.debug("gpu stop event is set or timeout")
